@@ -34,138 +34,140 @@
 #include "baobab-utils.h"
 #include "baobab-prefs.h"
 
-
-static gboolean props_changed;
-
-static GtkTreeView *tree_props;
-static GtkListStore *model_props;
-static GtkListStore *create_props_model (void);
-static GtkWidget *create_tree_props (GtkBuilder *builder);
-static void fill_props_model (GtkWidget *);
-static void check_toggled (GtkCellRendererToggle * cell,
-			   gchar * path_str, gpointer data);
-
-static void read_gconf (void);
-static void save_gconf (void);
-static gboolean set_gconf_list (GtkTreeModel * model, GtkTreePath * path,
-				GtkTreeIter * iter, gpointer data);
-
-static gboolean set_model_checks (GtkTreeModel * model, GtkTreePath * path,
-				  GtkTreeIter * iter, gpointer data);
-
-static void enable_home_cb (GtkToggleButton * togglebutton,
-			    gpointer user_data);
-
-
-void
-props_notify (GConfClient *client,
-	      guint cnxn_id,
-	      GConfEntry *entry,
-	      gpointer user_data)
+enum
 {
-	GSList *uris;
+	COL_CHECK,
+	COL_DEVICE,
+	COL_MOUNT_D,
+	COL_MOUNT,
+	COL_TYPE,
+	COL_FS_SIZE,
+	COL_FS_AVAIL,
+	TOT_COLUMNS
+};
 
-	uris = 	gconf_client_get_list (client,
-				       PROPS_SCAN_KEY,
-				       GCONF_VALUE_STRING,
-				       NULL);
-	baobab_set_excluded_locations (uris);
-	g_slist_foreach (uris, (GFunc) g_free, NULL);
-	g_slist_free (uris);
+static gboolean
+add_excluded_item (GtkTreeModel  *model,
+		   GtkTreePath   *path,
+		   GtkTreeIter   *iter,
+		   GSList       **list)
+{
+	GSList *l;
+	gchar *mount;
+	gboolean check;
+
+	l = *list;
+
+	gtk_tree_model_get (model,
+			    iter,
+			    COL_MOUNT, &mount,
+			    COL_CHECK, &check,
+			    -1);
+
+	if (!check) {
+		l = g_slist_prepend (l, mount);
+	}
+
+	*list = l;
+
+	return FALSE;
+}
+
+static GSList *
+get_excluded_locations (GtkTreeModel *model)
+{
+	GSList *l = NULL;
+
+	gtk_tree_model_foreach (model,
+				(GtkTreeModelForeachFunc) add_excluded_item,
+				&l);
+
+	return g_slist_reverse (l);
+}
+
+static void
+save_gconf (GtkTreeModel *model)
+{
+	GSList *l;
+
+	l = get_excluded_locations (model);
+
+	gconf_client_set_list (baobab.gconf_client,
+			       BAOBAB_EXCLUDED_DIRS_KEY, GCONF_VALUE_STRING,
+			       l, NULL);
+
+	g_slist_foreach (l, (GFunc) g_free, NULL);
+	g_slist_free (l);
+}
+
+static void
+enable_home_cb (GtkToggleButton *togglebutton, gpointer user_data)
+{
+	gboolean enable;
+
+	enable = gtk_toggle_button_get_active (togglebutton);
+
+	gconf_client_set_bool (baobab.gconf_client,
+			       BAOBAB_ENABLE_HOME_MONITOR_KEY,
+			       enable,
+			       NULL);
 
-	baobab_get_filesystem (&g_fs);
-	set_label_scan (&g_fs);
-	show_label ();
-	gtk_tree_store_clear (baobab.model);
-	first_row ();
 }
 
 static void
 filechooser_response_cb (GtkDialog *dialog,
-                         gint       response_id,
-                         gpointer   user_data)
+                         gint response_id,
+                         GtkTreeModel *model)
 {
 	switch (response_id) {
 		case GTK_RESPONSE_HELP:
 			baobab_help_display (GTK_WINDOW (baobab.window), 
 			                     "baobab.xml", "baobab-preferences");
 			break;
+		case GTK_RESPONSE_DELETE_EVENT:
 		case GTK_RESPONSE_CLOSE:
-			if (props_changed) { 
-				save_gconf (); 
-			}
+			save_gconf (model); 
 		default:
 			gtk_widget_destroy (GTK_WIDGET (dialog));
 			break;
 	}
 }
 
-void
-create_props (void)
+static void
+check_toggled (GtkCellRendererToggle *cell,
+	       gchar *path_str,
+	       GtkTreeModel *model)
 {
-	GtkWidget *dlg, *check_enablehome;
-	GtkBuilder *builder;
-	GError *error = NULL;
+	GtkTreeIter iter;
+	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
+	gboolean toggle;
+	gchar *mountpoint;
 
-	props_changed = FALSE;
+	/* get toggled iter */
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get (model,
+			    &iter,
+			    COL_CHECK, &toggle,
+			    COL_MOUNT_D, &mountpoint,
+			    -1);
 
-	/* UI stuff */
-	builder = gtk_builder_new ();
-	gtk_builder_add_from_file (builder, BAOBAB_DIALOG_SCAN_UI_FILE, &error);
+	/* check if root dir */
+	if (strcmp ("/", mountpoint) == 0)
+		goto clean_up;
 
-	if (error) {
-		g_critical ("Can't load user interface file for the scan properties dialog: %s",
-			    error->message);
-		g_object_unref (builder);
-		g_error_free (error);
+	/* set new value */
+	gtk_list_store_set (GTK_LIST_STORE (model),
+			    &iter,
+			    COL_CHECK, !toggle,
+			    -1);
 
-		return;
-	}
-
-	dlg = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_scan_props"));
-
-	gtk_window_set_transient_for (GTK_WINDOW (dlg),
-				      GTK_WINDOW (baobab.window));
-
-	tree_props = (GtkTreeView *) create_tree_props (builder);
-	fill_props_model (dlg);
-	read_gconf ();
-
-	check_enablehome = GTK_WIDGET (gtk_builder_get_object (builder, "check_enable_home"));
-	gtk_toggle_button_set_active ((GtkToggleButton *) check_enablehome,
-				      baobab.bbEnableHomeMonitor);
-
-	g_signal_connect_after ((GtkToggleButton *) check_enablehome,
-				"toggled", G_CALLBACK (enable_home_cb),
-				NULL);
-
-	g_signal_connect (dlg, "response",
-		    	  G_CALLBACK (filechooser_response_cb),
-		    	  NULL);
-
-  	gtk_widget_show_all (dlg);
+ clean_up:
+	g_free (mountpoint);
+	gtk_tree_path_free (path);
 }
 
-GtkListStore *
-create_props_model (void)
-{
-	GtkListStore *mdl;
-
-	mdl = gtk_list_store_new (TOT_COLUMNS,
-				  G_TYPE_BOOLEAN,	/* checkbox */
-				  G_TYPE_STRING,	/* device */
-				  G_TYPE_STRING,	/*mount point display */
-				  G_TYPE_STRING,	/* mount point uri */
-				  G_TYPE_STRING,	/* fs type */
-				  G_TYPE_STRING,	/* fs size */
-				  G_TYPE_STRING		/* fs avail */
-				  );
-
-	return mdl;
-}
-
-GtkWidget *
-create_tree_props (GtkBuilder *builder)
+static void
+create_tree_props (GtkBuilder *builder, GtkTreeModel *model)
 {
 	GtkCellRenderer *cell;
 	GtkTreeViewColumn *col;
@@ -176,7 +178,7 @@ create_tree_props (GtkBuilder *builder)
 	/* checkbox column */
 	cell = gtk_cell_renderer_toggle_new ();
 	g_signal_connect (cell, "toggled",
-			  G_CALLBACK (check_toggled), NULL);
+			  G_CALLBACK (check_toggled), model);
 
 	col = gtk_tree_view_column_new_with_attributes (_("Scan"), cell,
 							"active", COL_CHECK,
@@ -224,69 +226,35 @@ create_tree_props (GtkBuilder *builder)
 							COL_FS_AVAIL, NULL);
 	g_object_set (G_OBJECT (cell), "xalign", (gfloat) 1.0, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tvw), col);
-	
-	model_props = create_props_model ();
-	gtk_tree_view_set_model (GTK_TREE_VIEW (tvw),
-				 GTK_TREE_MODEL (model_props));
-	g_object_unref (model_props);
 
-	return tvw;
+	gtk_tree_view_set_model (GTK_TREE_VIEW (tvw), model);
+	g_object_unref (model);
 }
 
-void
-check_toggled (GtkCellRendererToggle *cell,
-	       gchar *path_str,
-	       gpointer data)
-{
-	GtkTreeIter iter;
-	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
-	gboolean toggle;
-	gchar *mountpoint;
-
-	/* get toggled iter */
-	gtk_tree_model_get_iter ((GtkTreeModel *) model_props, &iter,
-				 path);
-	gtk_tree_model_get ((GtkTreeModel *) model_props, &iter, COL_CHECK,
-			    &toggle, COL_MOUNT_D, &mountpoint, -1);
-
-	/* check if root dir */
-	if (strcmp ("/", mountpoint) == 0)
-		goto clean_up;
-
-	/* set new value */
-	props_changed = TRUE;
-	toggle ^= 1;
-	gtk_list_store_set (GTK_LIST_STORE (model_props), &iter, COL_CHECK,
-			    toggle, -1);
-
- clean_up:
-	g_free (mountpoint);
-	gtk_tree_path_free (path);
-}
-
-void
-fill_props_model (GtkWidget *dlg)
+static void
+fill_props_model (GtkListStore *store)
 {
 	GtkTreeIter iter;
 	guint lo;
 	glibtop_mountlist mountlist;
 	glibtop_mountentry *mountentry, *mountentry_tofree;
 	guint64 fstotal, fsavail;
-	
+
 	mountentry_tofree = glibtop_get_mountlist (&mountlist, 0);
 
-	for (lo = 0, mountentry = mountentry_tofree; lo < mountlist.number;
+	for (lo = 0, mountentry = mountentry_tofree;
+	     lo < mountlist.number;
 	     lo++, mountentry++) {
-	     
-	     	glibtop_fsusage fsusage;
+		glibtop_fsusage fsusage;
 		gchar * total, *avail;
-		GFile	*file;
-		gchar	*uri;
-		
-                struct stat buf;
-                if (g_stat(mountentry->devname,&buf) == -1)
-                  continue;		
-				
+		GFile *file;
+		gchar *uri;
+		gboolean excluded;
+
+		struct stat buf;
+		if (g_stat (mountentry->devname,&buf) == -1)
+			continue;
+
 		glibtop_get_fsusage (&fsusage, mountentry->mountdir);
 		fstotal = fsusage.blocks * fsusage.block_size;
 		fsavail = fsusage.bfree * fsusage.block_size;
@@ -294,16 +262,18 @@ fill_props_model (GtkWidget *dlg)
 		avail = g_format_size_for_display(fsavail);
 		file = g_file_new_for_path (mountentry->mountdir);
 		uri = g_file_get_uri (file);
-		gtk_list_store_append (model_props, &iter);
-			gtk_list_store_set (model_props, &iter,
-					    COL_CHECK, TRUE,
-					    COL_DEVICE, mountentry->devname,
-					    COL_MOUNT_D, mountentry->mountdir, 
-					    COL_MOUNT, uri, 
-					    COL_TYPE, mountentry->type,
-					    COL_FS_SIZE, total,
-					    COL_FS_AVAIL, avail,
-					    -1);
+		excluded = baobab_is_excluded_location (file);
+
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter,
+				    COL_CHECK, !excluded,
+				    COL_DEVICE, mountentry->devname,
+				    COL_MOUNT_D, mountentry->mountdir,
+				    COL_MOUNT, uri,
+				    COL_TYPE, mountentry->type,
+				    COL_FS_SIZE, total,
+				    COL_FS_AVAIL, avail,
+				    -1);
 		g_free(total);
 		g_free(avail);
 		g_free(uri);
@@ -314,104 +284,54 @@ fill_props_model (GtkWidget *dlg)
 }
 
 void
-read_gconf (void)
+baobab_prefs_dialog (void)
 {
-	GSList *l;
+	GtkBuilder *builder;
+	GtkWidget *dlg;
+	GtkWidget *check_enablehome;
+	GtkListStore *model;
+	GError *error = NULL;
 
-	l = gconf_client_get_list (baobab.gconf_client, PROPS_SCAN_KEY,
-				   GCONF_VALUE_STRING, NULL);
+	builder = gtk_builder_new ();
+	gtk_builder_add_from_file (builder, BAOBAB_DIALOG_SCAN_UI_FILE, &error);
+
+	if (error) {
+		g_critical ("Can't load user interface file for the scan properties dialog: %s",
+			    error->message);
+		g_object_unref (builder);
+		g_error_free (error);
 
-	if (!l)
 		return;
-
-	gtk_tree_model_foreach (GTK_TREE_MODEL (model_props),
-				set_model_checks, l);
-
-	g_slist_foreach (l, (GFunc) g_free, NULL);
-	g_slist_free (l);
-}
-
-void
-save_gconf (void)
-{
-	GSList *l = NULL;
-
-	l = g_slist_append (l, "START");
-	gtk_tree_model_foreach (GTK_TREE_MODEL (model_props),
-				set_gconf_list, l);
-	l = g_slist_remove (l, "START");
-	gconf_client_set_list (baobab.gconf_client,
-			       PROPS_SCAN_KEY, GCONF_VALUE_STRING,
-			       l, NULL);
-
-	g_slist_foreach (l, (GFunc) g_free, NULL);
-	g_slist_free (l);
-}
-
-gboolean
-set_gconf_list (GtkTreeModel *model,
-		GtkTreePath *path,
-		GtkTreeIter *iter,
-		gpointer list)
-{
-	gchar *mount;
-	gboolean check;
-
-	gtk_tree_model_get (model, iter, COL_MOUNT, &mount, COL_CHECK,
-			    &check, -1);
-
-	/* add only un-checked mount points */
-	if (!check) {
-		list = g_slist_last (list);
-		list = g_slist_append (list, mount);
 	}
 
-	/* g_free(mount); //freed in save_gconf() */
-	return FALSE;
+	dlg = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_scan_props"));
+
+	gtk_window_set_transient_for (GTK_WINDOW (dlg),
+				      GTK_WINDOW (baobab.window));
+
+	model = gtk_list_store_new (TOT_COLUMNS,
+				    G_TYPE_BOOLEAN,	/* checkbox */
+				    G_TYPE_STRING,	/* device */
+				    G_TYPE_STRING,	/*mount point display */
+				    G_TYPE_STRING,	/* mount point uri */
+				    G_TYPE_STRING,	/* fs type */
+				    G_TYPE_STRING,	/* fs size */
+				    G_TYPE_STRING	/* fs avail */
+				    );
+
+	create_tree_props (builder, GTK_TREE_MODEL (model));
+	fill_props_model (model);
+
+	check_enablehome = GTK_WIDGET (gtk_builder_get_object (builder, "check_enable_home"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_enablehome),
+				      baobab.monitor_home != NULL);
+
+	g_signal_connect (check_enablehome, "toggled",
+			  G_CALLBACK (enable_home_cb), NULL);
+
+	g_signal_connect (dlg, "response",
+			  G_CALLBACK (filechooser_response_cb), model);
+
+	gtk_widget_show_all (dlg);
 }
 
-static gint
-list_find (gconstpointer a, gconstpointer b)
-{
-	gchar *str_a, *str_b;
-	gint ret;
-	
-	str_a = g_uri_unescape_string (a,NULL);
-	str_b = g_uri_unescape_string (b,NULL);
-
-	ret = strcmp (str_a, str_b);
-
-	g_free (str_a);
-	g_free (str_b);
-
-	return ret;
-}
-
-gboolean
-set_model_checks (GtkTreeModel *model,
-		  GtkTreePath *path,
-		  GtkTreeIter *iter,
-		  gpointer list)
-{
-	gchar *mount;
-
-	gtk_tree_model_get (model, iter, COL_MOUNT, &mount, -1);
-
-	if (g_slist_find_custom (list, mount, list_find) != NULL)
-		gtk_list_store_set ((GtkListStore *) model, iter,
-				    COL_CHECK, FALSE, -1);
-
-	g_free (mount);
-
-	return FALSE;
-}
-
-void
-enable_home_cb (GtkToggleButton *togglebutton, gpointer user_data)
-{
-	baobab.bbEnableHomeMonitor = gtk_toggle_button_get_active (togglebutton);
-
-	gconf_client_set_bool (baobab.gconf_client, PROPS_ENABLE_HOME_MONITOR,
-			       baobab.bbEnableHomeMonitor, NULL);
-
-}
