@@ -32,7 +32,6 @@
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkbindings.h>
 #include <glib/gi18n.h>
 
 #include "gdict-sidebar.h"
@@ -146,15 +145,17 @@ gdict_sidebar_menu_position_function (GtkMenu  *menu,
 				      gpointer  user_data)
 {
   GtkWidget *widget;
+  GtkAllocation allocation;
 
   g_assert (GTK_IS_BUTTON (user_data));
 
   widget = GTK_WIDGET (user_data);
 
-  gdk_window_get_origin (widget->window, x, y);
+  gdk_window_get_origin (gtk_widget_get_window (widget), x, y);
 
-  *x += widget->allocation.x;
-  *y += widget->allocation.y + widget->allocation.height;
+  gtk_widget_get_allocation (widget, &allocation);
+  *x += allocation.x;
+  *y += allocation.y + allocation.height;
 
   *push_in = FALSE;
 }
@@ -165,13 +166,15 @@ gdict_sidebar_select_button_press_cb (GtkWidget      *widget,
 				      gpointer        user_data)
 {
   GdictSidebar *sidebar = GDICT_SIDEBAR (user_data);
+  GtkAllocation allocation;
 
   if (event->button == 1)
     {
       GtkRequisition req;
       gint width;
 
-      width = widget->allocation.width;
+      gtk_widget_get_allocation (widget, &allocation);
+      width = allocation.width;
       gtk_widget_set_size_request (sidebar->priv->menu, -1, -1);
       gtk_widget_size_request (sidebar->priv->menu, &req);
       gtk_widget_set_size_request (sidebar->priv->menu,
