@@ -806,7 +806,7 @@ logview_about (GtkWidget *widget, GtkWidget *window)
 static void
 logview_toggle_statusbar (GtkAction *action, LogviewWindow *logview)
 {
-  if (GTK_WIDGET_VISIBLE (logview->priv->statusbar))
+  if (gtk_widget_get_visible (logview->priv->statusbar))
     gtk_widget_hide (logview->priv->statusbar);
   else
     gtk_widget_show (logview->priv->statusbar);
@@ -815,7 +815,7 @@ logview_toggle_statusbar (GtkAction *action, LogviewWindow *logview)
 static void
 logview_toggle_sidebar (GtkAction *action, LogviewWindow *logview)
 {
-  if (GTK_WIDGET_VISIBLE (logview->priv->sidebar))
+  if (gtk_widget_get_visible (logview->priv->sidebar))
     gtk_widget_hide (logview->priv->sidebar);
   else
     gtk_widget_show (logview->priv->sidebar);
@@ -1011,10 +1011,7 @@ read_new_lines_cb (LogviewLog *log,
   gsize len;
 
   if (error != NULL) {
-    /* no new strings in the primary error message,
-     * as we're string frozen
-     */
-    primary = g_strdup_printf ("\"%s\":",
+    primary = g_strdup_printf (_("Can't read from \"%s\""),
                                logview_log_get_display_name (log));
     logview_window_add_error (window, primary, error->message);
     g_free (primary);
